package utils

import (
	"archive/tar"
	"bytes"
	"errors"
	"io"
	"io/ioutil"
	"os"
	"path/filepath"
	"strings"
)

//Tar tar a folder and return a reader?
func Tar(fpath string, f *os.File) (io.ReadWriteCloser, error) {
	var err error

	var w io.ReadWriteCloser
	if f == nil { // if not file we do memory
		w = NopRWCloser(&bytes.Buffer{})
	} else {
		w = f
	}

	if !filepath.IsAbs(fpath) {
		return nil, errors.New("Path must be absolute")
	}
	// Add slash to path end so files can be relative
	if fpath[len(fpath)-1] != '/' {
		fpath += "/"
	}

	tw := tar.NewWriter(w)
	err = filepath.Walk(fpath, func(p string, info os.FileInfo, err error) error {
		if info.IsDir() && info.Name()[0] == '.' {
			return filepath.SkipDir
		}
		if info.IsDir() || info.Name()[0] == '.' { // Skip file/dir
			return nil
		}

		npath := strings.TrimPrefix(p, fpath)
		theader, err := tar.FileInfoHeader(info, "")
		if err != nil {
			return err
		}
		theader.Name = npath // Relative path to root

		// Write file header
		err = tw.WriteHeader(theader)
		if err != nil {
			return err
		}

		f, err := os.Open(p)
		if err != nil {
			return err
		}

		_, err = io.Copy(tw, f)
		if err != nil {
			return err
		}
		return nil
	})
	if err != nil {
		return nil, err
	}
	tw.Flush()
	if f != nil {
		f.Seek(0, os.SEEK_SET)
	}

	return w, nil
}

//TempTar will delete temporary tar file on close
func TempTar(fpath string, tmppath string, prefix string) (io.ReadCloser, error) {

	tarFile, err := ioutil.TempFile(tmppath, prefix)
	if err != nil {
		return nil, err
	}

	tarReader, err := Tar(fpath, tarFile)
	if err != nil {
		return nil, err
	}

	return &tarCloseDelete{tarReader, tarFile.Name()}, nil

}

// Hepler to Delete tar on close
type tarCloseDelete struct {
	io.ReadCloser
	tarFileName string
}

func (t *tarCloseDelete) Close() error {
	err := t.ReadCloser.Close()
	if err != nil {
		return err
	}
	if t.tarFileName == "" {
		return nil
	}

	err = os.Remove(t.tarFileName)
	if err != nil {
		return err
	}
	t.tarFileName = "" // Prevents double remove
	// Remove tar file if necessary
	return nil
}
